{"cells": [{"cell_type": "markdown", "metadata": {}, "source": ["# A more complex system: H$_{2}$O\n", "\n", "We will now move to a bit more complex system, with more than one atom. We will focus on the geometry of the system and the effect of basis set used. \n", "\n", "Again, set up the environment and import the required modules:"]}, {"cell_type": "code", "execution_count": 1, "metadata": {}, "outputs": [{"name": "stdout", "output_type": "stream", "text": ["\n", "  Memory set to   1.863 GiB by Python driver.\n", "  Threads set to 2 by Python driver.\n"]}], "source": ["import psi4\n", "import pandas as pd\n", "import numpy as np\n", "import matplotlib.pyplot as plt\n", "import sys\n", "sys.path.append(\"..\")\n", "from helpers import *\n", "\n", "psi4.set_memory('2 GB')\n", "psi4.set_num_threads(2)"]}, {"cell_type": "markdown", "metadata": {}, "source": ["## Building the Geometry\n", "In the previous example the geometry file was quite trivial since we had a single atom. Now we will need to set up the geometry for a slightly more complicated system: a water molecule. \n", "\n", "Define the geometry for a water molecule in the next cell, making sure to use the right syntax. Remember that you can use either Z-matrix or Cartesian format (one of the two will make your life easier!) and that information about the total charge of the system and spin multiplicity also need to be provided. It may be useful to refer to the dropdown menu in the previous section about atomic coordinates representation."]}, {"cell_type": "code", "execution_count": null, "metadata": {}, "outputs": [], "source": ["h2o = psi4.geometry(\"\"\"\n", "## FILL HERE\n", "\"\"\")"]}, {"cell_type": "code", "execution_count": 3, "metadata": {}, "outputs": [{"data": {"application/3dmoljs_load.v0": "<div id=\"3dmolviewer_16655853984277906\"  style=\"position: relative; width: 400px; height: 400px\">\n        <p id=\"3dmolwarning_16655853984277906\" style=\"background-color:#ffcccc;color:black\">You appear to be running in JupyterLab (or JavaScript failed to load for some other reason).  You need to install the 3dmol extension: <br>\n        <tt>jupyter labextension install jupyterlab_3dmol</tt></p>\n        </div>\n<script>\n\nvar loadScriptAsync = function(uri){\n  return new Promise((resolve, reject) => {\n    var tag = document.createElement('script');\n    tag.src = uri;\n    tag.async = true;\n    tag.onload = () => {\n      resolve();\n    };\n  var firstScriptTag = document.getElementsByTagName('script')[0];\n  firstScriptTag.parentNode.insertBefore(tag, firstScriptTag);\n});\n};\n\nif(typeof $3Dmolpromise === 'undefined') {\n$3Dmolpromise = null;\n  $3Dmolpromise = loadScriptAsync('https://cdn.jsdelivr.net/npm/3dmol@latest/build/3Dmol-min.min.js');\n}\n\nvar viewer_16655853984277906 = null;\nvar warn = document.getElementById(\"3dmolwarning_16655853984277906\");\nif(warn) {\n    warn.parentNode.removeChild(warn);\n}\n$3Dmolpromise.then(function() {\nviewer_16655853984277906 = $3Dmol.createViewer($(\"#3dmolviewer_16655853984277906\"),{backgroundColor:\"white\"});\nviewer_16655853984277906.zoomTo();\n\tviewer_16655853984277906.addModel(\"3\\n\\n O    0.000000000000    0.000000000000   -0.066834625719\\n H    0.000000000000   -0.764370430999    0.530357005346\\n H    0.000000000000    0.764370430999    0.530357005346\\n\",\"xyz\");\n\tviewer_16655853984277906.setStyle({\"stick\": {}, \"sphere\": {\"scale\": 0.1}});\n\tviewer_16655853984277906.zoomTo();\nviewer_16655853984277906.render();\n});\n</script>", "text/html": ["<div id=\"3dmolviewer_16655853984277906\"  style=\"position: relative; width: 400px; height: 400px\">\n", "        <p id=\"3dmolwarning_16655853984277906\" style=\"background-color:#ffcccc;color:black\">You appear to be running in JupyterLab (or JavaScript failed to load for some other reason).  You need to install the 3dmol extension: <br>\n", "        <tt>jupyter labextension install jupyterlab_3dmol</tt></p>\n", "        </div>\n", "<script>\n", "\n", "var loadScriptAsync = function(uri){\n", "  return new Promise((resolve, reject) => {\n", "    var tag = document.createElement('script');\n", "    tag.src = uri;\n", "    tag.async = true;\n", "    tag.onload = () => {\n", "      resolve();\n", "    };\n", "  var firstScriptTag = document.getElementsByTagName('script')[0];\n", "  firstScriptTag.parentNode.insertBefore(tag, firstScriptTag);\n", "});\n", "};\n", "\n", "if(typeof $3Dmolpromise === 'undefined') {\n", "$3Dmolpromise = null;\n", "  $3Dmolpromise = loadScriptAsync('https://cdn.jsdelivr.net/npm/3dmol@latest/build/3Dmol-min.min.js');\n", "}\n", "\n", "var viewer_16655853984277906 = null;\n", "var warn = document.getElementById(\"3dmolwarning_16655853984277906\");\n", "if(warn) {\n", "    warn.parentNode.removeChild(warn);\n", "}\n", "$3Dmolpromise.then(function() {\n", "viewer_16655853984277906 = $3Dmol.createViewer($(\"#3dmolviewer_16655853984277906\"),{backgroundColor:\"white\"});\n", "viewer_16655853984277906.zoomTo();\n", "\tviewer_16655853984277906.addModel(\"3\\n\\n O    0.000000000000    0.000000000000   -0.066834625719\\n H    0.000000000000   -0.764370430999    0.530357005346\\n H    0.000000000000    0.764370430999    0.530357005346\\n\",\"xyz\");\n", "\tviewer_16655853984277906.setStyle({\"stick\": {}, \"sphere\": {\"scale\": 0.1}});\n", "\tviewer_16655853984277906.zoomTo();\n", "viewer_16655853984277906.render();\n", "});\n", "</script>"]}, "metadata": {}, "output_type": "display_data"}], "source": ["drawXYZ(h2o)"]}, {"cell_type": "markdown", "metadata": {}, "source": [":::{admonition} Exercise 9\n", ":class: exercise \n", "Include in your report the coordinates you set for your water molecules and a screenshot of how it looks like in 3D representation.\n", ":::"]}, {"cell_type": "markdown", "metadata": {}, "source": ["```{admonition} Exercise 10\n", ":class: exercise\n", "How do you calculate the spin multiplicity of a species? Compare the case of the water molecule to the previous example of the hydrogen atom.\n", "```"]}, {"cell_type": "markdown", "metadata": {}, "source": ["Extending what we have seen with the hydrogen atom, we will now investigate the effect of the choice of the basis sets when we have *heavy* atoms (remember that in the context of computational chemsitry, we consider as *heavy* each atom other than hydrogen)."]}, {"cell_type": "markdown", "metadata": {}, "source": [":::{admonition} Exercise 11\n", ":class: exercise \n", "Compute energy calculations for the different basis sets indicated below with a loop similar to the one used before.  Include a table of the the calculated energies. Specify what is the difference between the basis sets that we used.\n", "\n", "**Bonus:** which of the additional functions introduced with respect to the `6-31G` basis set are more reasonable for the present case, i.e. a water molecule? Why?\n", ":::"]}, {"cell_type": "code", "execution_count": null, "metadata": {}, "outputs": [], "source": ["basissets = ['6-31G', '6-311G', '6-31+G', '6-31G**', '6-31++G**'] # these are the basis sets we are going to use\n", "\n", "psi4.set_options({'reference':'UHF'}) # We are using Unrestriced Hartee Fock\n", "\n", "for basis in basissets:\n", "    psi4.core.set_output_file(f'{basis}-h2o-output.log', False) # save in seperate log files\n", "    E = None  # define here the energy calculation with psi4 for single point calculations\n", "\n", "    print(basis, E)"]}, {"cell_type": "markdown", "metadata": {}, "source": ["Few things important to keep in mind:\n", "* As you have seen, and will see in the next exercises, the value of total energy is highly dependent on the method and the basis set. In the hydrogen example, we were lucky to have an exact solution to compare with, but most time it is useless to discuss about total energy! What we are usually interested in are energies changes for the same systems with different states, or  binding energy.\n", "* All the calculations performed so far on the water molecule are referred to the geometry that you defined. By running such a single point energy calculation we find the lowest energy solution for the Schr\u00f6dinger equation at the **current** geometry. This however does not guarantee it is the lowest-energy conformation, i.e. our calculations may refer to a geometry in practice never observed in nature. What we usually do is performing a **geometry optimization**, i.e.  starting from a given geometry and arriving an optimal geometry. This is extremely important expecially when we are dealing with complex molecules, for which it is possible we don't know how the molecule looks like, for example not knowing the equilibrium bond length or the angle between the atoms. This concept will be used a lot in this course, and one of the sections of the next exercise session will focus on geometry optimization.\n", " "]}, {"cell_type": "markdown", "metadata": {"tags": []}, "source": ["# Output Files \n", "\n", "So far we only focused on the energy given as ouput from the `psi4.energy()` function. However, for each calculation a .log file is produced, containing detailed information of the procedure.\n", "\n", "You can list the files in your directory (by typing `ls` in your terminal) to see what new files have been generated. Indeed,\n", "you should find the `.log` files created by Psi4. As this files may be very large and we have no intention of editing it. \n", "\n", "As example, you can use the command `head` followed by the name of the last `.log` file created (`6-31++G*-output.log`) to see the last lines of that file. By using the option `-20`, the last 20 lines will be printed (where you can use any number instad of 50). Make sure to specify the file name as `6-31++G\\*\\*-h2o-output.log`, where the `*`needs to be preceded by a `\\` symbol. This is because otherwise in Linux syntax the `*` is a special character that will be interpreted as 'zero or more characters', i.e. by typing `6-31++G**-h2o-output.log` the system will look for all files which have zero or more characters between `6-31++G` and `-h2o-output.log`. However, in our case we want to use `*` as a proper character (which is the file name), that's the reason of using `\\*`. "]}, {"cell_type": "code", "execution_count": 11, "metadata": {}, "outputs": [{"name": "stdout", "output_type": "stream", "text": ["\n", "  Electronic Dipole Moment: [e a0]\n", "     X:     0.0000      Y:     0.0000      Z:    -0.0864\n", "\n", "  Dipole Moment: [e a0]\n", "     X:     0.0000      Y:     0.0000      Z:     0.9077     Total:     0.9077\n", "\n", "  Dipole Moment: [D]\n", "     X:     0.0000      Y:     0.0000      Z:     2.3071     Total:     2.3071\n", "\n", "\n", "*** tstop() called on noto.epfl.ch at Wed Oct 12 16:46:05 2022\n", "Module time:\n", "\tuser time   =       0.65 seconds =       0.01 minutes\n", "\tsystem time =       0.03 seconds =       0.00 minutes\n", "\ttotal time  =          0 seconds =       0.00 minutes\n", "Total time:\n", "\tuser time   =      18.78 seconds =       0.31 minutes\n", "\tsystem time =       0.91 seconds =       0.02 minutes\n", "\ttotal time  =        493 seconds =       8.22 minutes\n"]}], "source": ["!tail -20  6-31++G\\*\\*-h2o-output.log"]}, {"cell_type": "markdown", "metadata": {}, "source": ["If you are interested in extracting the lines of a file containing one particular keyword another useful command to know is `grep`.\n", "For example, considering again the last `.log` file created, it is possible to extract the final energy with the command:"]}, {"cell_type": "code", "execution_count": 12, "metadata": {}, "outputs": [{"name": "stdout", "output_type": "stream", "text": ["  @DF-UHF Final Energy:   -76.02966176454109\n"]}], "source": ["!grep 'Final Energy' 6-31++G\\*\\*-h2o-output.log"]}, {"cell_type": "markdown", "metadata": {}, "source": ["```{admonition} Exercise 12\n", ":class: exercise\n", "Have a look at the log files produced so far and answer the following questions:\n", "1.  What is the significance of the statement *Energy and wave function converged*?\n", "\n", "2.  What is the meaning of the different *iter* preceding *Energy and wave function converged*? \n", "    Compare the number of cycles for the different basis sets.\n", "```"]}], "metadata": {"celltoolbar": "Tags", "kernelspec": {"display_name": "Computational Chemistry", "language": "python", "name": "compchem"}, "language_info": {"codemirror_mode": {"name": "ipython", "version": 3}, "file_extension": ".py", "mimetype": "text/x-python", "name": "python", "nbconvert_exporter": "python", "pygments_lexer": "ipython3", "version": "3.8.12"}}, "nbformat": 4, "nbformat_minor": 4}